/***************************************************/
/* Node.js script for COUNTER-APP local web server */
/* =============================================== */
/*                                                 */
/* - Run this script and goto                      */
/*		http://localhost:3000/counter.html         */
/***************************************************/

var express = require('express');  // For directory based file server

var SerialPort = require("serialport").SerialPort;  // For bluetooth serial port access
var serialPort = new SerialPort(process.argv[2], {  // process.argv[2] gives the bluetooth incoming port (COM?), set from the command line
	baudrate: 115200  // Arduino program baudrate
});

var countVal = 0;  // Used for the counting each serialport incoming data

// Handling serial port events.
serialPort.on("open", function () {

    console.log('open');
    
	serialPort.on('data', function(data) {  // executed when data is available on serial port
		console.log('data received: ' + data);
		// Increments countVal for every value '1' from the serial port
		if(data == 1)		countVal = countVal+1;
    });

    serialPort.write("ls\n", function(err, results) {
	
	console.log('err ' + err);
	
	console.log('results ' + results);

	});

});

var app = express();  // file server application variable

app.get('/', function(req, res) { // Goto the root of the server to get this output 

    	res.send('This is the counter server');

});

app.get('/reset', function(req,res) {  // Used to reset the counter ( called when browser reloads page)

    	countVal = 0;
    
	res.send("Success");

});

app.get('/count',function(req,res) {  // Returns the count value

    	res.send(String(countVal));

});

app.use(express.static('public'));  // Serves files in the folder 'public' under the server root.

app.listen(3000, function() {  // Sets the local server with port 3000

    	console.log('Listening on port 3000');

});
